@@ -1,10 +1,45 @@
 #
-# 1. Save what valid positions we have
-# 2. Have a flag to indicate if dec clamp is in place
-#
 # $Name:  $
 #
 # $Log: Control.pm,v $
+# Revision 0.10  2005/04/17 04:21:43  robert
+# Bug fixes:
+#    - Change memory card reset from 0x300: 0x80 0x05 0x00 to
+#                                    0x300: 0x80 0x91
+#    - Fix _outb command to actually write data...  Allows above to
+#      do something.
+# Enhancements:
+#    - Return array of pictures taken from take_object_pictures,
+#      take_dark_pictures and take_bias_pictures.
+#
+# Revision 0.9  2005/03/31 03:40:55  robert
+# Fix the fixed non-working comment implementation
+# Add DEBUG log statement when using ra_to_limit
+#
+# Revision 0.8  2005/03/22 03:08:19  robert
+# Fix the initial non-working comment implementation
+# Fix the doc of init_telescope to be more clear
+#
+# Revision 0.7  2005/03/13 23:11:22  robert
+#    Config file:
+#         - Add support for signed/unsigned fits option with downfits
+#         - Add support for ADC channel selection for chiller control
+#         - Add support for specifying digits after day for fits file names
+#         - Add support for turning off automatic clearing of CCD's
+#         - Add support for putting comments in the FITS header of images
+#         - Add support for specifying RA range
+#    Methods:
+#         - Add option to init_telescope to not home mechanisms
+#    Bug fixes:
+#         - Fix not checking for ability to write config and position files
+#         - Fix CRVAL1 and CRVAL2 fits comment fields (reversed)
+#         - Fix error seen when CLAMP was at position 0 but not used
+#         - Fix ra_move_to_degree to actually check if destination can be reached
+#
+# Revision 0.6  2005/02/11 02:33:31  robert
+# Add support for signed/unsigned fits values.
+# Add support for specifying ADC used for chiller control.
+#
 # Revision 0.5  2005/01/16 03:21:09  robert
 # Switch from Time::Object to Date::Time
 # Add option to ignore maservo errors
@@ -56,7 +91,7 @@ use DateTime;
 use IO::Socket;
 use IO::Select;
 
-my $ID = q$Id: Control.pm,v 0.5 2005/01/16 03:21:09 robert Exp $;
+my $ID = q$Id: Control.pm,v 0.10 2005/04/17 04:21:43 robert Exp $;
 my $version = join( ' ', ( split ( ' ', $ID ) )[ 1 .. 3 ] );
 $version =~ s/,v\b//;
 $version =~ s/(\S+)$/($1)/;
@@ -75,9 +110,9 @@ require Log::Agent::Tag::Callback;
 
 BEGIN{ use_ok( 'TASS::Control' ); }
 
-my $no_hash = TASS::Control->new;
-ok( defined $no_hash, 'defined from new with no hash' );
-isa_ok( $no_hash, 'TASS::Control', 'ISA TASS::Control' );
+#my $no_hash = TASS::Control->new( './tass-control.cfg' );
+#ok( defined $no_hash, 'defined from new with no hash' );
+#isa_ok( $no_hash, 'TASS::Control', 'ISA TASS::Control' );
 #
 # Set up the logging
 unlink 'td.err', 'td.dbg', 'td.out';
@@ -90,9 +125,9 @@ logconfig( -driver => Log::Agent::Driver::File->make(
            -level => INFO
          );
 
-my $hash = TASS::Control->new( undef, ( 'port' => '/dev/ttyS0',
-                                        'baud' => '9600' ) );
-
+#my $hash = TASS::Control->new( undef, ( 'port' => '/dev/ttyS0',
+#                                        'baud' => '9600' ) );
+#
 #$hash->start;
 
 =end testing
@@ -191,7 +226,7 @@ use Class::MethodMaker
    boolean       => [ qw/_driving_to_limit/ ],
    #
    # Chiller information
-   boolean       => [ qw/chiller_use/ ],
+   boolean       => [ qw/chiller_use temp_adc/ ],
    get_set       => [ qw/max_time_on min_time/ ],
    get_set       => [ qw/duty_cycle _chiller_last_time/ ],
    get_set       => [ qw/delta_temp min_temp dead_band/ ],
@@ -253,7 +288,9 @@ use Class::MethodMaker
    get_set       => [ qw/dark_diff max_object_dark_time/ ],
    counter       => [ qw/_bias_count _dark_count _object_count/ ],
    counter       => [ qw/_bad_transfer _overexposure/ ],
-   get_set       => [ qw/equinox file_suffix/ ];
+   get_set       => [ qw/equinox file_suffix fits_signed digits_after_day/ ],
+   get_set       => [ qw/auto_clear comment/ ],
+   list          => [ qw/comments/ ];
 
 sub _setup_var
    {
@@ -294,11 +331,11 @@ sub init
    {
    my $self = shift;
    my $ini_file = shift || 'tass-control.cfg';
+   die "Cannot write $ini_file" unless -w $ini_file;
    my %args = @_;
    my %ini;
    my @settings;
 
-
    if ( -e $ini_file )
       {
       tie %ini, 'Config::IniFiles', ( -file => $ini_file );
@@ -352,8 +389,8 @@ sub init
       'CRPIX2', 'Pixel coordinate of reference point Y axis',
       'CD1_1',  'Conversion matrix element',
       'CD2_2',  'Conversion matrix element',
-      'CRVAL1', 'Galactic longitude at reference point X axis',
-      'CRVAL2', 'Galactic latitude at reference point Y axis'
+      'CRVAL1', 'Galactic latitude at reference point X axis',
+      'CRVAL2', 'Galactic longitude at reference point Y axis'
       );
 
    #
@@ -395,6 +432,9 @@ sub init
       [ qw/site telescop/, 'ROB', 'Telescope name' ],
 
       [ qw/chiller chiller_use 1/, '1/0 if using a chiller or not' ],
+      [ qw/chiller temp_adc 4/,
+        [ 'ADC channel the thermometer is attached to.  Can be air temp,',
+          'wet bulb, etc...' ] ],
       [ qw/chiller duty_cycle 50/,
         'Max time chiller will be run, decimal percent' ],
       [ qw/chiller max_time_on 900/,
@@ -402,7 +442,7 @@ sub init
       [ qw/chiller min_time 300/,
         'Minimum time chiller will be on or off at a time in seconds' ],
       [ qw/chiller delta_temp 15/,
-        'The desired air/water temperature delta' ],
+        'The desired adc/water temperature delta' ],
       [ qw/chiller min_temp 5/,
         'The minimum temperature to cool the water to' ],
       [ qw/chiller dead_band 2/,
@@ -410,8 +450,17 @@ sub init
 
       [ qw/camera naxis1 2064/ ],
       [ qw/camera naxis2 2037/ ],
+      [ qw/camera digits_after_day 4/,
+          [ '4 will give a file name of kir_31791646 (the day is the first',
+	    'four digits, 3179)' ] ],
       [ qw/camera file_suffix .fits/,
           'The desired suffix for the FITS files' ],
+      [ qw/camera fits_signed 0/,
+          [ 'Whether the fits files have signed values (1) or not (0)',
+            '(-32677 to 32768) or (0 to 65536)' ] ],
+      [ qw/camera auto_clear 1/,
+        [ 'Whether to automatically clear the CCD\'s when either',
+          'dark_diff or max_object_dark_time times are excedded' ], ],
       [ qw/camera dark_diff 10/,
         [ 'Subtracted from the dark exposure time to determine the',
           'maximum time lapsed from the last exposre to not execute',
@@ -422,29 +471,37 @@ sub init
         [ 'The maximum extra shutter close time from the last picture end',
           'before a clearing scan is executed.  Only affects the dark time',
           'of the first exposure in a set, not the shutter open time' ], ],
+      [ qw/camera comments ./,
+        [ 'Comments to put in the fits header.  For multiple lines, try:',
+          'comments=<<EOC',
+          'first comment line',
+          'second comment line',
+          'EOC' ], ],
       [ qw/camera camera_0_filter V/, 'Filter for camera 0' ],
       [ qw/camera camera_1_filter I/, 'Filter for camera 1' ],
       [ qw/camera focus_0_range 1500/, 'Focus range for camera 0' ],
       [ qw/camera focus_1_range 1300/, 'Focus range for camera 0' ],
-      [ qw/camera focus_0_best 550/, 'Best focus position for camera 0' ],
-      [ qw/camera focus_1_best 300/, 'Best focus position for camera 1' ],
-      [ qw/camera shutter_0_open_pos 220/, 'Absolute servo position' ],
+      [ qw/camera focus_0_best 520/, 'Best focus position for camera 0' ],
+      [ qw/camera focus_1_best 360/, 'Best focus position for camera 1' ],
+      [ qw/camera shutter_0_open_pos 230/, 'Absolute servo position' ],
       [ qw/camera shutter_0_close_pos 40/, 'Absolute servo position' ],
-      [ qw/camera shutter_1_open_pos 230/, 'Absolute servo position' ],
+      [ qw/camera shutter_1_open_pos 240/, 'Absolute servo position' ],
       [ qw/camera shutter_1_close_pos 55/, 'Absolute servo position' ],
 
       [ qw/telescope position_file position_file.ini/,
         'Stores the positions of the motors after program exit' ],
-      [ qw/telescope dec_drive_steps_degree/, 1627 / 13.5 ],
-      [ qw/telescope dec_drive_level_from_limit 150/,
+      [ qw/telescope dec_drive_steps_degree/, 90.6 ],
+      [ qw/telescope dec_drive_level_from_limit 160/,
         'Steps from limit, optics horizontal' ],
-      [ qw/telescope dec_drive_range 20170/, 'Steps' ],
+      [ qw/telescope dec_drive_range 13700/, 'Steps' ],
       [ qw/telescope dec_drive_backlash 1/, 'Steps' ],
       [ qw/telescope dec_clamp_use 0/, 'Whether to use clamp or not' ],
       [ qw/telescope dec_clamp_in 400/, 'Steps' ],
       [ qw/telescope dec_clamp_out 400/, 'Steps' ],
-      [ qw/telescope ra_level_from_limit 3438/,
+      [ qw/telescope ra_level_from_limit 4584/,
         'Siderial seconds from meridian to limit' ],
+      [ qw/telescope ra_range 9170/,
+        'Siderial seconds of total range of travel' ],
       [ qw/telescope ra_max_speed 32/,
         'Maximum speed to drive RA at - must be power of 2' ],
 
@@ -494,10 +551,10 @@ sub init
       {
       $self->_setup_var( \%ini, @$_ );
       }
-
    #
-   # In seconds
-   $self->ra_range( $self->ra_level_from_limit * 2 );
+   # Treat the comments differently
+   #$self->comments( split( m:$/:, $ini{ 'camera' }{ 'comments' } ) );
+   $self->comments( @{$ini{ 'camera' }{ 'comments' }} );
 
    $self->_hash_init( %args );
    #
@@ -587,7 +644,8 @@ sub start
       $self->driver->load_register( 'MPX', 0 );
       #
       # Reset the memory card per Service note 4
-      $self->_outb( '0x300', '0x80', '0x05', '0x00' );
+      #$self->_outb( '0x300', '0x80', '0x05', '0x00' );
+      $self->_outb( '0x300', '0x80', '0x91' );
       $self->driver->pulse( 17 );
       }
    child
@@ -612,11 +670,16 @@ sub start
 
 =over
 
-=item C<$tc-E<gt>init_telescope>
+=item C<$tc-E<gt>init_telescope( SCALAR )>
 
-Initializes the telescope for use.  Turns off all motors, closes the dec
-clamp, drives the dec motor to it's limit, drives the ra motor to it's limit,
-closes the camera shutters.
+Initializes the telescope for use.  Turns off all motors, closes the
+shutters and turns off the chiller.  If 1 is specified,
+all motors will be driven to their limits to determine their position if
+their current position is not known (position.ini file).  When
+0 is specified, the motor positions that are unknown will be set to 0.
+The default is 1 (find home when needed).
+
+B<Note:>Use '0' with caution as the position file may be incorrect.
 
 =item I<For Example>
 
@@ -624,6 +687,8 @@ closes the camera shutters.
 
  $tc->init_telescope;
 
+ $tc->init_telescope( 0 );
+
 =for example end
 
 =back
@@ -635,6 +700,7 @@ closes the camera shutters.
 sub init_telescope
    {
    my $self = shift;
+   my $move = shift || 1;
    #
    # Make sure all the motors that are on will be off
    $self->all_off;
@@ -654,7 +720,8 @@ sub init_telescope
    if ( !defined $self->ra_position )
       {
       $self->ra_position( 0 );
-      $self->ra_to_limit( );
+      $self->ra_to_limit( )
+         if $move;
       }
    #
    #
@@ -674,7 +741,8 @@ sub init_telescope
       if ( !defined $self->$position )
          {
          $self->$position( 0 );
-         $self->$to_limit;
+         $self->$to_limit
+            if $move;
          }
       }
    #
@@ -694,7 +762,6 @@ sub _load_positions
    {
    my $self = shift;
 
-
    if ( -e $self->position_file )
       {
       my %ini;
@@ -723,6 +790,7 @@ sub _save_positions
 
    if ( -e $self->position_file )
       {
+      die "Cannot write $self->position_file" unless -w $self->position_file;
       tie %ini, 'Config::IniFiles', ( -file => $self->position_file );
       }
    else
@@ -1048,7 +1116,7 @@ sub dec_drive_move_steps
    $self->dec_drive_position( $self->dec_drive_position + $steps );
    #
    # Close the clamp
-   $self->dec_clamp_close if $clamped && $self->dec_clamp_used;
+   $self->dec_clamp_close if $clamped && $self->dec_clamp_use;
    #
    # make sure the motor if off
 #   $status &= $self->driver->motor_off( 'DEC_DRIVE' )
@@ -1575,7 +1643,8 @@ sub ra_to_limit
    my $sidereal = 0;
    my $time;
    #
-   #
+   # Log our current position for debugging purposes
+   logtrc DEBUG, "RA position at ra_to_limit %.2f", $self->ra_current_second;
    if ( $self->ra_current_second < 0 )
       {
       $self->ra_on( abs( $speed ) );
@@ -2032,14 +2101,24 @@ sub ra_move_to_degree
    $ra_time = abs( $ra_seconds );
    #
    # If we're wrapping...
-   if ( $ra_time > (2 * $self->ra_level_from_limit) )
+   if ( $ra_time > $self->ra_range )
       {
       $ra_seconds += 360 * 3600 / 15 if ( $ra_seconds < 0 );
       $ra_seconds -= 360 * 3600 / 15 if ( $ra_seconds > 0 );
       }
-   logdbg INFO, sprintf "RA Move %f at %d",
-                $ra_seconds, $speed;
-   $self->ra_move_second( $ra_seconds, $speed );
+   if ( ($ra_seconds < 0) or ($ra_seconds < $self->ra_range) )
+      {
+      logdbg INFO, sprintf "Asked to move to RA out of range %f at %d",
+                   $ra_seconds, $speed;
+      return 0;
+      }
+   else
+      {
+      logdbg INFO, sprintf "RA Move %f at %d",
+                   $ra_seconds, $speed;
+      $self->ra_move_second( $ra_seconds, $speed );
+      return 1;
+      }
    }
 
 =over
@@ -2583,7 +2662,7 @@ sub get_file_names
    #
    # To generate the file name, we generate the integer portion...
    my $file_base = int(  ( $self->_picture_time - 50000 )
-                       * 10000 + 0.5 );
+                       * 10 ** $self->digits_after_day + 0.5 );
    #
    # ...then we append the file suffix...
    $file_base .= $self->file_suffix;
@@ -2610,13 +2689,11 @@ sub _outb
    {
    my $self = shift;
 
-#   my $port = shift;
-#   my $command;
    my $command = 'outb ' . join ' ', @_;
    logdbg INFO, 'Calling outb: ' . join ' ', @_;
-#   map { 'outb $port $_' } @_;
-#   for @_ { 'outb $port $_' };
-#   return `$command`;
+   my $result = `$command`;
+   chomp $result;
+   return $result;
    }
 
 sub _wait_for_transfer
@@ -2835,8 +2912,8 @@ sub _download_pictures
    my $comment;
    my $status = 0;
    logdbg INFO, "$id: Downloading %s and %s", @names;
-   my $cmd = sprintf "downfits -n %d -m %d -c %s -d %s",
-                      $self->naxis1, $self->naxis2, @names;
+   my $cmd = sprintf "downfits -n %d -m %d -s %d -c %s -d %s",
+                      $self->naxis1, $self->naxis2, $self->fits_signed, @names;
 
    system $cmd;
    use Astro::FITS::CFITSIO qw( :longnames :constants );
@@ -2871,6 +2948,13 @@ sub _download_pictures
                              $self->pic_info_str( $_ ),
                              defined $comment ? $comment : undef, $status );
       }
+
+   for my $comment ( $self->comments )
+      {
+      $fptr0->write_comment( $comment, $status );
+      $fptr1->write_comment( $comment, $status );
+      }
+
    $fptr0->write_key_str( 'FILTER',
                           $self->camera_0_filter,
                           undef, $status );
@@ -2940,7 +3024,7 @@ sub _wait_for_download
 
 =over
 
-=item C<$tc-E<gt>take_object_pictures( SCALAR, SCALAR, SCALAR, SCALAR )>
+=item C<ARRAY=$tc-E<gt>take_object_pictures( SCALAR, SCALAR, SCALAR, SCALAR )>
 
 Takes object pictures.  The first two arguments are mandatory and specify
 the number of pictures to take and their desired exposure length.  The
@@ -2956,14 +3040,17 @@ being dictated only by the time it takes to execute the given moves.
 The last argument is the 'ra_jitter' flag, and indicates if the ra
 drive should be turned off during the picture transfer (48.9 seconds) or not.
 
+Returns an array of array references that are the pairs of image file names
+taken.
+
 =item I<For Example>
 
 =for example begin
 
- $tc->take_object_pictures( 10, 60 );
- $tc->take_object_pictures( 10, 60, undef, 4 );
- $tc->take_object_pictures( 10, 60, -30, undef );
- $tc->take_object_pictures( 10, 100, -50, -1 );
+ @images = $tc->take_object_pictures( 10, 60 );
+ @images = $tc->take_object_pictures( 10, 60, undef, 4 );
+ @images = $tc->take_object_pictures( 10, 60, -30, undef );
+ @images = $tc->take_object_pictures( 10, 100, -50, -1 );
 
 =for example end
 
@@ -2978,7 +3065,7 @@ sub take_object_pictures
    my $self = shift;
    my( $count, $duration, $ra, $dec, $ra_jitter ) = @_;
 
-   my $time;
+   my( $time, @pictures );
 
    #
    # If the last picture ended real recently, we don't
@@ -2991,7 +3078,8 @@ sub take_object_pictures
    else
       {
       $self->_wait_for_download;
-      $self->clear_ccds;
+      $self->clear_ccds
+         if $self->auto_clear;
       }
 
    for ( 1..$count )
@@ -3068,7 +3156,9 @@ sub take_object_pictures
       $self->_object_count_incr;
 
       $self->_download( "O $_" );
+      push @pictures, [ $self->get_file_names ];
       }
+   return @pictures;
    }
 
 =over
@@ -3077,16 +3167,19 @@ sub take_object_pictures
 
 =over
 
-=item C<$tc-E<gt>take_dark_pictures( SCALAR, SCALAR )>
+=item C<ARRAY=$tc-E<gt>take_dark_pictures( SCALAR, SCALAR )>
 
 Takes dark images.  The first argument specifies the image count and the
 second the desired exposure length.
 
+Returns an array of array references that are the pairs of image file names
+taken.
+
 =item I<For Example>
 
 =for example begin
 
- $tc->take_dark_pictures( 2, 60 );
+ @images = $tc->take_dark_pictures( 2, 60 );
 
 =for example end
 
@@ -3101,7 +3194,7 @@ sub take_dark_pictures
    my $self = shift;
    my( $count, $duration ) = @_;
 
-   my $time;
+   my( $time, @pictures );
 
    #
    # If the last picture ended in within a good time,
@@ -3117,7 +3210,8 @@ sub take_dark_pictures
    else
       {
       $self->_wait_for_download;
-      $self->clear_ccds;
+      $self->clear_ccds
+         if $self->auto_clear;
       }
 
    $self->pic_info_num( 'darktime', 0 );
@@ -3165,7 +3259,9 @@ sub take_dark_pictures
       $self->_dark_count_incr;
 
       $self->_download( "D $_" );
+      push @pictures, [ $self->get_file_names ];
       }
+   return @pictures;
    }
 
 =over
@@ -3174,16 +3270,19 @@ sub take_dark_pictures
 
 =over
 
-=item C<$tc-E<gt>take_bias_pictures( SCALAR )>
+=item C<ARRAY=$tc-E<gt>take_bias_pictures( SCALAR )>
 
 Takes bias images.  The argument specifies the number of bias images to
 take.
 
+Returns an array of array references that are the pairs of image file names
+taken.
+
 =item I<For Example>
 
 =for example begin
 
- $tc->take_bias_pictures( 1 );
+ @images = $tc->take_bias_pictures( 1 );
 
 =for example end
 
@@ -3198,7 +3297,7 @@ sub take_bias_pictures
    my $self = shift;
    my( $count ) = @_;
 
-   my $time;
+   my( $time, @pictures );
 
    $self->pic_info_num( 'darktime', 0 );
 
@@ -3232,7 +3331,9 @@ sub take_bias_pictures
 
       $self->_download( "B $_" );
       $self->chiller_control;
+      push @pictures, [ $self->get_file_names ];
       }
+   return @pictures;
    }
 
 =over
@@ -3351,7 +3452,7 @@ sub chiller_control
    return if !$self->chiller_use;
 
    my $water_temp_adc = 0;
-   my $air_temp_adc = 4;
+   my $air_temp_adc = $self->temp_adc;
 
    my( $water_status, $water_temp )
       = $self->driver->read_adc( $water_temp_adc );
